@@ -11,7 +11,13 @@ if uname | grep -i openbsd ; then
 else
   # Use these for testing on Linux and Mac OSX:
   WARNING_OPTS="-Wshadow -Wall -Wextra -pedantic-errors -Wno-long-long"
-  WARNING_ANDROID_OPTS="-Wshadow -Wall -Wextra"
+  gccvers=`gcc -v 2>&1 | grep '^gcc version'`
+  if test "$gccvers" '<' 'gcc version 6'
+  then
+    WARNING_ANDROID_OPTS="-Wshadow -Wall -Wextra"
+  else
+    WARNING_ANDROID_OPTS="-Wshadow -Wall -Wextra -Wimplicit-fallthrough=0"
+  fi
 fi
 
 rm -f sqlite3.c
@@ -20,13 +26,12 @@ echo '********** No optimizations.  Includes FTS4/5, GEOPOLY, JSON1 ***'
 echo '**********    ' Options: $WARNING_OPTS
 gcc -c $WARNING_OPTS -std=c89 \
       -ansi -DHAVE_STDINT_H -DSQLITE_ENABLE_FTS4 -DSQLITE_ENABLE_GEOPOLY \
-      -DSQLITE_ENABLE_FTS5 -DSQLITE_ENABLE_JSON1 \
+      -DSQLITE_ENABLE_FTS5 \
       sqlite3.c
 if test x`uname` = 'xLinux'; then
 echo '********** Android configuration ******************************'
 echo '**********    ' Options: $WARNING_ANDROID_OPTS
 gcc -c \
-  -DHAVE_USLEEP=1 \
   -DSQLITE_HAVE_ISNAN \
   -DSQLITE_DEFAULT_JOURNAL_SIZE_LIMIT=1048576 \
   -DSQLITE_THREADSAFE=2 \
@@ -56,5 +61,5 @@ echo '********** Optimized -O3.  Includes FTS4/5, GEOPOLY, JSON1 ******'
 echo '**********    ' Options: $WARNING_OPTS
 gcc -O3 -c $WARNING_OPTS -std=c89 \
       -ansi -DHAVE_STDINT_H -DSQLITE_ENABLE_FTS4 -DSQLITE_ENABLE_GEOPOLY \
-      -DSQLITE_ENABLE_FTS5 -DSQLITE_ENABLE_JSON1 \
+      -DSQLITE_ENABLE_FTS5 \
       sqlite3.c
